<?php
/*
Plugin Name: WP-Footnotes
Plugin URI: http://www.elvery.net/drzax/more-things/wordpress-footnotes-plugin/
Version: 2.2
Description: Allows a user to easily add footnotes to a post.
Author: Simon Elvery
Author URI: http://www.elvery.net/drzax/
*/

/*
 * This file is part of WP-Footnotes a plugin for Word Press
 * Copyright (C) 2007 Simon Elvery
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 

// Some important constants
define('WP_FOOTNOTES_OPEN', " ((");  //You can change this if you really have to, but I wouldn't recommend it.
define('WP_FOOTNOTES_CLOSE', "))");  //Same with this one.
define('WP_FOOTNOTES_VERSION', '2.2');

// Define default options
$wp_footnotes_default_options = array('superscript'=>true,
									  'smooth_scroll'=>false,
									  'pre_backlink'=>' [',
									  'backlink'=>'&#8617;',
									  'post_backlink'=>']',
									  'pre_identifier'=>'',
									  'list_style_type'=>'decimal',
									  'post_identifier'=>'',
									  'pre_footnotes'=>'',
									  'post_footnotes'=>'',
									  'style_rules'=>'ol.footnotes{font-size:0.8em; color:#666666;}',
									  'no_display_home'=>false,
									  'no_display_archive'=>false,
									  'no_display_date'=>false,
									  'no_display_category'=>false,
									  'no_display_search'=>false,
									  'no_display_feed'=>false,
									  'combine_identical_notes'=>false,
									  'priority'=>10,
									  'version'=>WP_FOOTNOTES_VERSION);

// Get the current settings or setup some defaults if needed
if (!$wp_footnotes_current_settings = get_option('swas_footnote_options')){
	swas_set_default_options($wp_footnotes_default_options);
} else { 
	// Set any unset options
	if ($wp_footnotes_current_settings['version'] != WP_FOOTNOTES_VERSION) {
		foreach ($wp_footnotes_default_options as $wp_footnotes_key => $wp_footnotes_value) {
			$wp_footnotes_change = false;
			if (!isset($wp_footnotes_current_settings[$wp_footnotes_key])) {
				$wp_footnotes_change = true;
				$wp_footnotes_current_settings[$wp_footnotes_key] = $wp_footnotes_value;
			}
			if ($wp_footnotes_change) update_option('swas_footnote_options', $wp_footnotes_current_settings);
		}
	}	
}

if (!empty($_POST['save_options'])){
	swas_save_options();
}elseif(!empty($_POST['reset_options'])){
	swas_set_default_options($wp_footnotes_default_options);
}

// Include number conversion if we need it
if ($wp_footnotes_current_settings['list_style_type'] != 'decimal') {require_once('num2char.php');}

/**
 * Sets all options to the default.
 */
function swas_set_default_options ($options){
	update_option('swas_footnote_options', '');
	update_option('swas_footnote_options', $options);
}

/**
 * Set options which aren't set yet.
 */
function swas_set_unset_options () {
	
}

/**
 * Searches the text and extracts footnotes. 
 * Adds the superscrip links and creats footnotes list.
 * @param $data string The content of the post.
 * @return string The new content with footnotes generated.
 */
function swas_footnote($data) {
	global $post, $wp_footnotes_current_settings;
	
	// Check for and setup the starting number
	$start_number = (preg_match("|<!\-\-startnum=(\d+)\-\->|",$data,$start_number_array)==1) ? $start_number_array[1] : 1;

	// Let's attempt this with Regex instead
	preg_match_all("/(".preg_quote(WP_FOOTNOTES_OPEN)."|<footnote>)(.*)(".preg_quote(WP_FOOTNOTES_CLOSE)."|<\/footnote>)/Us", $data, $identifiers, PREG_SET_ORDER);
	
	// Check whether we are displaying them or not
	$display = true;
	if ($wp_footnotes_current_settings['no_display_home'] && is_home()) $display = false;
	if ($wp_footnotes_current_settings['no_display_archive'] && is_archive()) $display = false;
	if ($wp_footnotes_current_settings['no_display_date'] && is_date()) $display = false;
	if ($wp_footnotes_current_settings['no_display_category'] && is_category()) $display = false;
	if ($wp_footnotes_current_settings['no_display_search'] && is_search()) $display = false;
	if ($wp_footnotes_current_settings['no_display_feed'] && is_feed()) $display = false;
	
	$footnotes = array();
	
	// Create 'em
	if (count($identifiers)>0){
		
		for ($i=0; $i<count($identifiers); $i++){
			// Look for ref: and replace in identifiers array.
			if (substr($identifiers[$i][2],0,4) == 'ref:'){
				$ref = (int)substr($identifiers[$i][2],4);
				$identifiers[$i]['text'] = $identifiers[$ref-1][2];
			}else{
				$identifiers[$i]['text'] = $identifiers[$i][2];
			}
			
			// if we're combining identical notes check if we've already got one like this & record keys
			if ($wp_footnotes_current_settings['combine_identical_notes']){
				for ($j=0; $j<count($footnotes); $j++){
					if ($footnotes[$j]['text'] == $identifiers[$i]['text']){
						$identifiers[$i]['use_footnote'] = $j;
						$footnotes[$j]['identifiers'][] = $i;
						break;
					}
				}
			}
			if (!isset($identifiers[$i]['use_footnote'])){
				// Add footnote and record the key
				$identifiers[$i]['use_footnote'] = count($footnotes);
				$footnotes[count($footnotes)]['text'] = $identifiers[$i]['text'];
				$footnotes[$identifiers[$i]['use_footnote']]['identifiers'][] = $i;
			}
		}
		
		foreach ($identifiers as $key => $value) {
			$id_id = "identifier_".$key."_".$post->ID;
			$id_num = ($wp_footnotes_current_settings['list_style_type'] != 'decimal') ? swas_convert_num($value['use_footnote']+$start_number, $wp_footnotes_current_settings['list_style_type']) : $value['use_footnote']+$start_number;
			$id_href = "#footnote_".$value['use_footnote']."_".$post->ID;
			$id_title = strip_tags($value['text']);
			if (is_feed()){
				$id_replace = $wp_footnotes_current_settings['pre_identifier'].$id_num.$wp_footnotes_current_settings['post_identifier'];
			}else{
				$id_replace = $wp_footnotes_current_settings['pre_identifier'].'<a href="'.$id_href.'" id="'.$id_id.'" class="footnote-link footnote-identifier-link" title="'.$id_title.'">'.$id_num.'</a>'.$wp_footnotes_current_settings['post_identifier'];
			}
			if ($wp_footnotes_current_settings['superscript']) $id_replace = '<sup>'.$id_replace.'</sup>';
			if ($display) $data = substr_replace($data, $id_replace, strpos($data,$value[0]),strlen($value[0]));
			else $data = substr_replace($data, '', strpos($data,$value[0]),strlen($value[0]));
		}
		
		if ($display) {
			$start = ($start_number != 1) ? 'start="'.$start_number.'" ' : '';
			$data = $data.$wp_footnotes_current_settings['pre_footnotes'].'<ol '.$start.'class="footnotes">';
			foreach ($footnotes as $key => $value) {
				$data = $data.'<li id="footnote_'.$key.'_'.$post->ID.'" class="footnote">';
				$data = $data.$value['text'];
				if (!is_feed()){
					foreach($value['identifiers'] as $identifier){
						$data = $data.$wp_footnotes_current_settings['pre_backlink'].'<a href="#identifier_'.$identifier.'_'.$post->ID.'" class="footnote-link footnote-back-link">'.$wp_footnotes_current_settings['backlink'].'</a>'.$wp_footnotes_current_settings['post_backlink'];
					}
				}
				$data = $data.'</li>';
			}
			$data = $data.'</ol>'.$wp_footnotes_current_settings['post_footnotes'];
		}
		//print_r($identifiers);
		//print_r($footnotes);
		
	}
    return $data;
}
function swas_smooth_scroll(){
	?>
<script type="text/javascript" language="javascript" src="http://yui.yahooapis.com/2.2.0/build/yahoo-dom-event/yahoo-dom-event.js"></script>
<script type="text/javascript" language="javascript" src="<?php echo get_option('siteurl');?>/wp-content/plugins/wp-footnotes/smooth_scroll.js"></script>
<?php
}

function swas_footnotes_options_page() { 
	$wp_footnotes_current_settings = get_option('swas_footnote_options');
	require_once('admin_panel.php');
}

function swas_add_options() {
	// Add a new menu under Options:
	add_options_page('Footnotes', 'Footnotes', 8, __FILE__, 'swas_footnotes_options_page');
}

function swas_save_options() {
	$footnotes_options['superscript'] = (array_key_exists('superscript', $_POST)) ? true : false;
	$footnotes_options['smooth_scroll'] = (array_key_exists('smooth_scroll', $_POST)) ? true : false;		

	$footnotes_options['pre_backlink'] = $_POST['pre_backlink'];
	$footnotes_options['backlink'] = $_POST['backlink'];
	$footnotes_options['post_backlink'] = $_POST['post_backlink'];
	
	$footnotes_options['pre_identifier'] = $_POST['pre_identifier'];
	$footnotes_options['list_style_type'] = $_POST['list_style_type'];
	$footnotes_options['post_identifier'] = $_POST['post_identifier'];

	$footnotes_options['pre_footnotes'] = stripslashes($_POST['pre_footnotes']);
	$footnotes_options['post_footnotes'] = stripslashes($_POST['post_footnotes']);
	$footnotes_options['style_rules'] = stripslashes($_POST['style_rules']);
	
	$footnotes_options['no_display_home'] = (array_key_exists('no_display_home', $_POST)) ? true : false;
	$footnotes_options['no_display_archive'] = (array_key_exists('no_display_archive', $_POST)) ? true : false;
	$footnotes_options['no_display_date'] = (array_key_exists('no_display_date', $_POST)) ? true : false;
	$footnotes_options['no_display_category'] = (array_key_exists('no_display_category', $_POST)) ? true : false;
	$footnotes_options['no_display_search'] = (array_key_exists('no_display_search', $_POST)) ? true : false;
	$footnotes_options['no_display_feed'] = (array_key_exists('no_display_feed', $_POST)) ? true : false;
	
	$footnotes_options['combine_identical_notes'] = (array_key_exists('combine_identical_notes', $_POST)) ? true : false;
	$footnotes_options['priority'] = $_POST['priority'];
	
	update_option('swas_footnote_options', $footnotes_options);
}

function swas_upgrade_post($data){
	$data = str_replace('<footnote>',WP_FOOTNOTES_OPEN,$data);
	$data = str_replace('</footnote>',WP_FOOTNOTES_CLOSE,$data);
	return $data;
}

function insert_styles(){
	global $wp_footnotes_current_settings;
	?>
	<style type="text/css">
		ol.footnotes li{list-style-type:<?php echo $wp_footnotes_current_settings['list_style_type']; ?>;}
		<?php echo $wp_footnotes_current_settings['style_rules'];?>
	</style>
	<?php
}

add_action('the_content', 'swas_footnote', $wp_footnotes_current_settings['priority']);
add_action('admin_menu', 'swas_add_options'); 		// Insert the Admin panel.
add_action('wp_head', 'insert_styles');
if ($wp_footnotes_current_settings['smooth_scroll']) add_action('wp_head', 'swas_smooth_scroll');
?>